import os
import time
from plico.utils.base_runner import BaseRunner
from plico.utils.logger import Logger
from plico.utils.decorator import override
from plico.utils.control_loop import FaultTolerantControlLoop
from palpao_server.mirror_controller.simulated_deformable_mirror import \
    SimulatedDeformableMirror
from palpao_server.mirror_controller.deformable_mirror_controller import \
    DeformableMirrorController
from plico.rpc.zmq_ports import ZmqPorts
from palpao.calibration.calibration_manager import CalibrationManager
from palpao_server.mirror_controller.bmc_deformable_mirror import \
    BmcDeformableMirror
from palpao_server.mirror_controller.alpao_deformable_mirror import \
    AlpaoDeformableMirror
import sys



class Runner(BaseRunner):

    RUNNING_MESSAGE = "Mirror controller is running."

    def __init__(self):
        BaseRunner.__init__(self)


    def _tryGetDefaultFlatTag(self):
        try:
            mirrorDeviceSection= self.configuration.getValue(
                self.getConfigurationSection(), 'mirror')
            return self.configuration.getValue(
                mirrorDeviceSection, 'default_flat_tag')
        except KeyError as e:
            self._logger.warn(str(e))
            return None


    def _createDeformableMirrorDevice(self):
        mirrorDeviceSection= self.configuration.getValue(
            self.getConfigurationSection(), 'mirror')
        mirrorModel= self.configuration.deviceModel(mirrorDeviceSection)
        if mirrorModel == 'simulatedMEMSMultiDM':
            self._createSimulatedDeformableMirror(mirrorDeviceSection)
        elif mirrorModel == 'simulatedDM':
            self._createSimulatedDeformableMirror(mirrorDeviceSection)
        elif mirrorModel == 'alpaoDM':
            self._createAlpaoMirror(mirrorDeviceSection)
        elif mirrorModel == 'piTipTilt':
            self._createPITipTiltMirror(mirrorDeviceSection)
        elif mirrorModel == 'bmc':
            self._createBmcDeformableMirror(mirrorDeviceSection)
        else:
            raise KeyError('Unsupported mirror model %s' % mirrorModel)


    def _createSimulatedDeformableMirror(self, mirrorDeviceSection):
        dmSerialNumber= self.configuration.getValue(
            mirrorDeviceSection, 'serial_number')
        self._mirror= SimulatedDeformableMirror(dmSerialNumber)


    def _createAlpaoMirror(self, mirrorDeviceSection):
        serialNumber= self.configuration.getValue(mirrorDeviceSection,
                                                  'serial_number')
        self._logger.notice("Creating ALPAO device SN %s" % serialNumber)
        libFolder= self.configuration.getValue(mirrorDeviceSection,
                                               'lib_folder')
        sys.path.append(libFolder)
        from asdk import DM
        alpaoDm= DM(serialNumber)
        self._mirror= AlpaoDeformableMirror(alpaoDm, serialNumber)


    def _createBmcDeformableMirror(self, mirrorDeviceSection):
        serialNumber= self.configuration.getValue(mirrorDeviceSection,
                                                  'serial_number')
        self._logger.notice("Creating BMC device SN %s" % serialNumber)
        import bmc
        bmcDm= bmc.BmcDm()
        self._logger.notice("BMC version <%s>" % bmcDm.version_string())
        self._mirror= BmcDeformableMirror(bmcDm, serialNumber)


    def _createPITipTiltMirror(self, mirrorDeviceSection):
        from palpao_server.mirror_controller.pi_tip_tilt_mirror \
            import PhysikInstrumenteTipTiltMirror
        from pi_gcs.gcs2 import GeneralCommandSet2
        from pi_gcs.tip_tilt_2_axes import TipTilt2Axis

        hostname= self.configuration.getValue(
            mirrorDeviceSection, 'ip_address')
        serialNumber= self.configuration.getValue(mirrorDeviceSection,
                                                  'serial_number')
        cfg= self._calibrationManager.loadPiTipTiltCalibration(
            serialNumber)
        cfg.hostname= hostname
        gcs=GeneralCommandSet2()
        tt=TipTilt2Axis(gcs, cfg)
        tt.setUp()
        self._mirror= PhysikInstrumenteTipTiltMirror(
            serialNumber, tt)


    def _createCalibrationManager(self):
        calibrationRootDir= self.configuration.calibrationRootDir()
        self._calibrationManager= CalibrationManager(calibrationRootDir)


    def _setUp(self):
        self._logger= Logger.of("Deformable Mirror Controller runner")

        self._zmqPorts= ZmqPorts.fromConfiguration(
            self.configuration, self.getConfigurationSection())
        self._replySocket = self.rpc().replySocket(
            self._zmqPorts.SERVER_REPLY_PORT)
        self._statusSocket = self.rpc().publisherSocket(
            self._zmqPorts.SERVER_STATUS_PORT)

        self._logger.notice('reply socket on port %d' %
                            self._zmqPorts.SERVER_REPLY_PORT)
        self._logger.notice('status socket on port %d' %
                            self._zmqPorts.SERVER_STATUS_PORT)

        self._createCalibrationManager()

        self._createDeformableMirrorDevice()

        flatFileTag= self._tryGetDefaultFlatTag()

        self._controller= DeformableMirrorController(
            self.name,
            self._zmqPorts,
            self._mirror,
            self._replySocket,
            self._statusSocket,
            self.rpc(),
            self._calibrationManager,
            flatFileTag)


    def _runLoop(self):
        self._logRunning()

        FaultTolerantControlLoop(
            self._controller,
            Logger.of("Deformable Mirror Controller control loop"),
            time,
            0.001).start()
        self._logger.notice("Terminated")


    @override
    def run(self):
        self._setUp()
        self._runLoop()
        return os.EX_OK


    @override
    def terminate(self, signal, frame):
        self._controller.terminate()



#!/usr/bin/env python

from plico.utils.decorator import override
from plico.utils.logger import Logger
from palpao_server.mirror_controller.abstract_deformable_mirror import \
    AbstractDeformableMirror


class AlpaoError(Exception):
    """Exception raised for ALPAO error.

    Attributes:
        errorCode -- BMC error code
        message -- explanation of the error
    """

    def __init__(self, errorCode, message):
        self.errorCode = errorCode
        self.message = message



class AlpaoDeformableMirror(AbstractDeformableMirror):


    def __init__(self, dm, serialNumber):
        self._dm= dm
        self._serialNumber= serialNumber
        self._logger= Logger.of('ALPAO Deformable Mirror')
        self._dm.Reset()


    @override
    def setZonalCommand(self, zonalCommand):
        self._dm.Send(zonalCommand)
        self._lastZonalCommand= zonalCommand


    @override
    def getZonalCommand(self):
        return self._lastZonalCommand


    @override
    def serialNumber(self):
        return self._serialNumber


    def getVersion(self):
        return int(self._dm.Get('VersionInfo'))


    @override
    def getNumberOfActuators(self):
        return int(self._dm.Get('NbOfActuator'))


    @override
    def deinitialize(self):
        self._dm.Stop()
        self._dm.Reset()




#!/usr/bin/env python
import unittest
import numpy as np
from palpao_server.mirror_controller.bmc_deformable_mirror import \
    BmcDeformableMirror, BmcError
from palpao_server.mirror_controller.fake_bmc_dm import FakeBmcDm



class BmcDeformableMirrorTest(unittest.TestCase):


    def setUp(self):
        self._serialNumber= FakeBmcDm.VALID_SERIAL_NUMBER
        self._bmcDm= FakeBmcDm()
        self._dm= BmcDeformableMirror(self._bmcDm,
                                      self._serialNumber)


    def tearDown(self):
        self._dm.deinitialize()


    def testCreateWithInvalidSerialNumberRaises(self):
        invalidSerial= 'asdasdf'
        self.assertRaises(
            BmcError, BmcDeformableMirror, self._bmcDm, invalidSerial)


    def testGetNumberOfActuators(self):
        self.assertEqual(
            self._bmcDm.NUMBER_OF_ACTUATORS,
            self._dm.getNumberOfActuators())


    def testSetZonalCommandWithWrongSizeRaises(self):
        wrongNumberOfActuators= self._dm.getNumberOfActuators() + 2
        zonalCommand= np.zeros(wrongNumberOfActuators)
        self.assertRaises(
            BmcError,
            self._dm.setZonalCommand, zonalCommand)


    def testSetAndGetZonalCommand(self):
        zonalCommand= np.arange(self._dm.getNumberOfActuators())
        self._dm.setZonalCommand(zonalCommand)
        actualCommand= self._dm.getZonalCommand()
        self.assertTrue(np.allclose(actualCommand, zonalCommand))


if __name__ == "__main__":
    unittest.main()






class Constants:
    METER_2_NANOMETER= 1e9
    APP_NAME= "inaf.arcetri.ao.palpao_server"
    APP_AUTHOR= "INAF Arcetri Adaptive Optics"
    THIS_PACKAGE= 'palpao_server'

    PROCESS_MONITOR_CONFIG_SECTION= 'processMonitor'
    DEFORMABLE_MIRROR_1_CONFIG_SECTION= 'server1'
    DEFORMABLE_MIRROR_2_CONFIG_SECTION= 'server2'

    # TODO: must be the same of console_scripts in setup.py
    START_PROCESS_NAME= 'palpao_start'
    STOP_PROCESS_NAME= 'palpao_stop'
    KILL_ALL_PROCESS_NAME= 'palpao_kill_all'
    MIRROR_CONTROLLER_1_PROCESS_NAME= 'palpao_mirror_controller_1'
    MIRROR_CONTROLLER_2_PROCESS_NAME= 'palpao_mirror_controller_2'



#!/usr/bin/env python
import unittest
from palpao_server.mirror_controller.simulated_deformable_mirror \
    import SimulatedDeformableMirror



class SimulatedDeformableMirrorTest(unittest.TestCase):


    def setUp(self):
        serialNumber= '1123'
        self._dm= SimulatedDeformableMirror(serialNumber)


    def tearDown(self):
        pass


    def testNumberOfActuator(self):
        self.assertEqual(SimulatedDeformableMirror.NUMBER_OF_ACTUATORS,
                         self._dm.getNumberOfActuators())


if __name__ == "__main__":
    unittest.main()



from plico.utils.decorator import override
from plico.utils.logger import Logger
from palpao_server.mirror_controller.abstract_deformable_mirror import \
    AbstractDeformableMirror


class PhysikInstrumenteTipTiltMirror(AbstractDeformableMirror):


    def __init__(self, serialNumber, tipTiltMirror):
        self._logger= Logger.of('PI TT Mirror')
        self._serialNumber= serialNumber
        self._tt= tipTiltMirror
        self._tt.stopModulation()
        self._tt.disableControlLoop()


    @override
    def setZonalCommand(self, zonalCommand):
        self._tt.setOpenLoopValue(zonalCommand)


    @override
    def getZonalCommand(self):
        return self._tt.getOpenLoopValue()


    @override
    def serialNumber(self):
        return self._serialNumber


    @override
    def deinitialize(self):
        pass


    @override
    def getNumberOfActuators(self):
        return 2



import numpy as np
from plico.utils.logger import Logger
from plico.utils.decorator import override
import scipy.signal.signaltools
from palpao_server.mirror_controller.abstract_modulator import \
    AbstractModulator

__version__= "$Id: pi_S334_modulator.py 278 2017-06-12 06:06:05Z pygo $"


class PIS334Modulator(AbstractModulator):

    ADAPTIVE_MODULATION_NONE= 'None'
    ADAPTIVE_MODULATION_ELLIPSE= 'Ellipse'
    ADAPTIVE_MODULATION_FREEFORM= 'Freeform'

    def __init__(self, name, tipTiltMirror):
        AbstractModulator.__init__(self)
        self._name= name
        self._tt= tipTiltMirror

        self._radiusInMilliRad= 1.
        self._frequencyInHz= 20.
        self._centerInMilliRad= np.array([0, 0.])

        self._logger= Logger.of("PI S334 Modulator")
        self._logger.notice('Initialized')

        self._currentRadii= np.zeros(2)
        self._currentCenter= np.zeros(2)
        self._currentPhases= np.zeros(2)
        self._wantedTrajectory= None

        self._nAdaptiveIterationEllipse= 3
        self._nAdaptiveIterationFreeform= 3
        self._adaptiveModulationMethod= self.ADAPTIVE_MODULATION_ELLIPSE


    def useNoAdaptiveModulation(self):
        self._adaptiveModulationMethod= self.ADAPTIVE_MODULATION_NONE


    def useEllipticalAdaptiveModulation(self):
        self._adaptiveModulationMethod= self.ADAPTIVE_MODULATION_ELLIPSE


    def useFreeformAdaptiveModulation(self):
        self._adaptiveModulationMethod= self.ADAPTIVE_MODULATION_FREEFORM


    def _applyChanges(self):
        if self._adaptiveModulationMethod == self.ADAPTIVE_MODULATION_NONE:
            self._modulate()
        elif self._adaptiveModulationMethod == \
                self.ADAPTIVE_MODULATION_ELLIPSE:
            self._modulateWithAdaptiveEllipse()
        elif self._adaptiveModulationMethod == \
                self.ADAPTIVE_MODULATION_FREEFORM:
            self._modulateWithAdaptiveFreeform()
        else:
            raise Exception(
                "Programming error. Unknown modulation method %s" % str(
                    self._adaptiveModulationMethod))


    def _modulate(self):
        self._wantedTrajectory= self._circle(
            [self._radiusInMilliRad, self._radiusInMilliRad],
            self._frequencyInHz,
            [0, 0],
            self._centerInMilliRad,
            self._tt.getRecordedDataTimeStep())
        self._applyTrajectory(self._wantedTrajectory)


    @override
    def name(self):
        return self._name


    @override
    def setRadiusInMilliRad(self, radiusInMilliRad):
        self._radiusInMilliRad = radiusInMilliRad
        self._applyChanges()


    @override
    def getRadiusInMilliRad(self):
        return self._radiusInMilliRad


    @override
    def setFrequencyInHz(self, frequencyInHz):
        self._frequencyInHz = frequencyInHz
        self._applyChanges()


    @override
    def getFrequencyInHz(self):
        return self._frequencyInHz


    @override
    def setCenterInMilliRad(self, center):
        self._centerInMilliRad= center
        self._applyChanges()


    @override
    def getCenterInMilliRad(self):
        return self._centerInMilliRad


    def _modulateWithAdaptiveEllipse(self):
        self._modulate()
        self._currentRadii= [self._radiusInMilliRad, self._radiusInMilliRad]
        self._currentCenter= self._centerInMilliRad
        self._currentPhases= np.zeros(2)

        for _ in range(self._nAdaptiveIterationEllipse):
            (useRadii, useCenter, usePhases)= self._computeAdaptedParameters(
                [self._radiusInMilliRad, self._radiusInMilliRad],
                self._centerInMilliRad)
            self._startModulation(useRadii, self._frequencyInHz,
                                  usePhases, useCenter)
            self._logger.notice(
                "radii want/set: %s / %s" % (
                    [self._radiusInMilliRad, self._radiusInMilliRad],
                    self._currentRadii))
            self._logger.notice("center want/set: %s / %s" %
                                (self._centerInMilliRad, self._currentCenter))
            self._logger.notice("phases set: %s" % usePhases)
            # self._plotError()


    def _modulateWithAdaptiveFreeform(self):
        self._modulateWithAdaptiveEllipse()

        for _ in range(self._nAdaptiveIterationFreeform):
            trajectoryError= self._getError()
            applyCommand= self._lastAppliedTrajectory + trajectoryError
            self._applyTrajectory(applyCommand)
            # self._plotError()


    def _applyTrajectory(self, trajectory):
        self._tt.startFreeformModulation(trajectory[0], trajectory[1])
        self._lastAppliedTrajectory= trajectory


    def _startModulation(self, radii, freq, phases, center):
        trajectory= self._circle(radii, freq, phases, center,
                                 self._tt.getRecordedDataTimeStep())
        self._applyTrajectory(trajectory)
        self._currentRadii= radii
        self._currentCenter= center
        self._currentPhases= phases


    def _getTrajectory(self):
        diagn= self.getDiagnosticData()
        return diagn[1:3]


    def _computeAdaptedParameters(self, wantRadii, wantCenter):
        actualTrajectory= self._getTrajectory()

        dt= self._tt.getRecordedDataTimeStep()
        periodInPoints= round(1./ self._frequencyInHz / dt)
        phaseShiftX, phaseShiftY= self._estimatePhaseShift(actualTrajectory,
                                                           periodInPoints)
        corrPhases= np.array([0, (phaseShiftY - phaseShiftX)])
        gotRadii= np.ptp(actualTrajectory, axis=1) / 2.0
        corrRadii= gotRadii / wantRadii
        gotCenter= 0.5 * (np.max(actualTrajectory, axis=1) +
                          np.min(actualTrajectory, axis=1))
        corrCenter= gotCenter - wantCenter

        self._logger.notice("got radii %s, corr %s" % (gotRadii, corrRadii))
        self._logger.notice("got center %s, corr %s" % (gotCenter, corrCenter))

        return (self._currentRadii / corrRadii,
                self._currentCenter - corrCenter,
                self._currentPhases - corrPhases)


    @override
    def getDiagnosticData(self):
        import time
        self._tt.getRecordedData(4)
        time.sleep(0.2)
        diagn= self._tt.getRecordedData(4000)
        return np.vstack((diagn, self._wantedTrajectory))


    def _findShiftWithAutoCorrelation(self, wanted, measured, periodInPoints):
        wantedZeroMean= wanted - np.mean(wanted)
        measuredZeroMean= measured - np.mean(measured)
        corr= scipy.signal.signaltools.correlate(
            wantedZeroMean[0:periodInPoints],
            measuredZeroMean,
            mode='valid')
        return np.argmax(corr[0: periodInPoints])


    def _estimatePhaseShift(self, measuredTrajectory, periodInPoints):
        shiftX= self._findShiftWithAutoCorrelation(
            self._wantedTrajectory[0], measuredTrajectory[0], periodInPoints)
        shiftY= self._findShiftWithAutoCorrelation(
            self._wantedTrajectory[1], measuredTrajectory[1], periodInPoints)
        self._logger.notice("shift in points %d %d " % (shiftX, shiftY))
        phaseShiftX= shiftX / periodInPoints * np.pi * 2
        phaseShiftY= shiftY / periodInPoints * np.pi * 2
        return (phaseShiftX, phaseShiftY)


    def _getError(self):
        measuredTraj= self._getTrajectory()
        dt= self._tt.getRecordedDataTimeStep()
        periodInPoints= round(1./ self._frequencyInHz / dt)
        phaseShiftX, phaseShiftY= self._estimatePhaseShift(measuredTraj,
                                                           periodInPoints)

        (xd, yd)= self._circle([self._radiusInMilliRad,
                                self._radiusInMilliRad],
                               self._frequencyInHz,
                               [phaseShiftX, phaseShiftY],
                               self._centerInMilliRad,
                               dt)
        diffX= xd - measuredTraj[0]
        diffY= yd - measuredTraj[1]
        # diffX= np.roll(diffX, shiftX)
        # diffY= np.roll(diffY, shiftY)
        error= np.array([diffX, diffY])
        self._logger.notice("Error max %f min %f rms %f" % (
            error.max(), error.min(), error.std()))
        return error


    def getResidualRms(self):
        error= self._getError()
        errorRms= np.mean(np.linalg.norm(error, axis=0))
        return errorRms


    def _circle(self, radiusInMilliRad, frequencyInHz,
                dephaseInRadians, centerInMilliRad, dt):
        t= np.linspace(0, dt* 4000, 4000, endpoint=False)
        x= radiusInMilliRad[0] * \
            np.cos(t* frequencyInHz* 2* np.pi + dephaseInRadians[0]) + \
            centerInMilliRad[0]
        y= radiusInMilliRad[1] * \
            np.sin(t* frequencyInHz* 2* np.pi + dephaseInRadians[1]) + \
            centerInMilliRad[1]
        return np.array([x, y])


    def _sine(self, radiusInMilliRad, frequencyInHz,
              phaseInRadians, centerInMilliRad, dt):
        t= np.linspace(0, dt* 4000, 4000, endpoint=False)
        x= radiusInMilliRad * np.sin(
            t* frequencyInHz* 2* np.pi + phaseInRadians) + centerInMilliRad
        return x

    def _plotError(self):
        import matplotlib.pyplot as plt
        plt.subplot(211)
        err= self._getError()
        plt.plot(err[0], label='error axis A')
        plt.plot(err[1], label='error axis B')
        plt.legend()
        plt.subplot(212)
        traj= self._wantedTrajectory
        actualTraj= self._getTrajectory()
        plt.plot(traj[0], traj[1], 'b', label='wanted')
        plt.plot(actualTraj[0], actualTraj[1], 'r', label='got')
        plt.legend()
        plt.show()



import threading
import numpy as np
from plico.utils.logger import Logger
from plico.utils.decorator import override, synchronized
from plico.utils.timekeeper import TimeKeeper
from plico.utils.stepable import Stepable
from plico.utils.snapshotable import Snapshotable
from plico.utils.hackerable import Hackerable
from palpao.types.deformable_mirror_status import DeformableMirrorStatus
from palpao.client.abstract_deformable_mirror_client import SnapshotEntry
from plico.utils.serverinfoable import ServerInfoable





class DeformableMirrorController(Stepable, Snapshotable, Hackerable,
                                 ServerInfoable):

    def __init__(self,
                 servername,
                 ports,
                 deformableMirror,
                 replySocket,
                 statusSocket,
                 rpcHandler,
                 calibrationManager,
                 flatZonalCommandTag):
        self._mirror= deformableMirror
        self._replySocket= replySocket
        self._statusSocket= statusSocket
        self._rpcHandler= rpcHandler
        self._calMgr= calibrationManager
        self._flatTag= flatZonalCommandTag
        self._logger= Logger.of('DeformableMirrorController')
        Hackerable.__init__(self, self._logger)
        ServerInfoable.__init__(self, servername,
                                ports,
                                self._logger)
        self._isTerminated= False
        self._stepCounter= 0
        self._commandCounter= 0
        self._timekeep = TimeKeeper()
        self._mirrorStatus= None
        self._mutexStatus= threading.RLock()
        self._modalCommand= None
        self._modalBasis= None
#        self._setModalBasis('zonal')
        self._initialize()
        self._logger.notice('Deformable Mirror Controller created')


    def _initialize(self):
        self.setFlatCommand(self._flatTag)
        self.setShape(np.zeros(self._getNumberOfModes()))


    @override
    def step(self):
        self._rpcHandler.handleRequest(self, self._replySocket, multi=True)
        self._publishStatus()
        if self._timekeep.inc():
            self._logger.notice(
                'Stepping at %5.2f Hz' % (self._timekeep.rate))
        self._stepCounter+= 1


    def _getStepCounter(self):
        return self._stepCounter


    def setFlatCommand(self, flatTag):
        self._flatCmd= self._calMgr.loadZonalCommand(flatTag)
        self._flatTag= flatTag


    def getFlatTag(self):
        return self._flatTag


    def terminate(self):
        self._logger.notice("Got request to terminate")
        try:
            self._mirror.deinitialize()
        except Exception as e:
            self._logger.warn("Could not deinitialize mirror: %s" %
                              str(e))
        self._isTerminated= True


    @override
    def isTerminated(self):
        return self._isTerminated


    def _getNumberOfActuators(self):
        return self._mirror.getNumberOfActuators()


    def _getNumberOfModes(self):
#        self._modalBasis.shape[1]
        return self._getNumberOfActuators()


#    @synchronized("_mutexStatus")
#    def _setModalBasis(self, modalBasisTag):
#        if modalBasisTag == 'zonal':
#            self._modalBasis= np.identity(self._getNumberOfActuators())
#            self._modalBasisTag= modalBasisTag

    def _convertModalToZonal(self, modalCommand):
        return modalCommand.copy()


    def _convertZonalToModal(self, zonalCommand):
        return zonalCommand.copy()


    def setShape(self, modalAmplitudes):
        zonalCommand= self._convertModalToZonal(modalAmplitudes)
        self._mirror.setZonalCommand(zonalCommand + self._flatCmd)
        self._modalCommand= modalAmplitudes.copy()
        self._commandCounter+= 1
        with self._mutexStatus:
            self._mirrorStatus= None


    def getShape(self):
        zonalCmd= self._mirror.getZonalCommand() - self._flatCmd
        return self._convertZonalToModal(zonalCmd)


    def getSnapshot(self, prefix):
        status= self._getMirrorStatus()
        snapshot= {}
        snapshot[SnapshotEntry.COMMAND_COUNTER]= status.commandCounter()
        snapshot[SnapshotEntry.SERIAL_NUMBER]= \
            self._getDeformableMirrorSerialNumber()
        snapshot[SnapshotEntry.STEP_COUNTER]= self._getStepCounter()
        return Snapshotable.prepend(prefix, snapshot)


    def _getDeformableMirrorSerialNumber(self):
        return self._mirror.serialNumber()



    @synchronized("_mutexStatus")
    def _getMirrorStatus(self):
        if self._mirrorStatus is None:
            self._logger.debug('get MirrorStatus')
            self._mirrorStatus= DeformableMirrorStatus(
                self._getNumberOfActuators(),
                self._getNumberOfModes(),
                self._mirror.getZonalCommand(),
                self._commandCounter)
            self._logger.debug(self._mirrorStatus)
        return self._mirrorStatus


    def _publishStatus(self):
        self._rpcHandler.publishPickable(self._statusSocket,
                                         self._getMirrorStatus())



#!/usr/bin/env python
import sys
from palpao_server.mirror_controller.runner import Runner
from plico.utils.config_file_manager import ConfigFileManager
from palpao_server.utils.constants import Constants

__version__ = "$Id: palpao_mirror_controller_2.py 30 2018-01-27 10:18:23Z lbusoni $"



def main():
    runner= Runner()
    configFileManager= ConfigFileManager(Constants.APP_NAME,
                                         Constants.APP_AUTHOR,
                                         Constants.THIS_PACKAGE)
    configFileManager.installConfigFileFromPackage()
    argv= ['', configFileManager.getConfigFilePath(),
           Constants.DEFORMABLE_MIRROR_2_CONFIG_SECTION]
    sys.exit(runner.start(argv))


if __name__ == '__main__':
    main()



#!/usr/bin/env python
import sys
from palpao_server.mirror_controller.runner import Runner

__version__ = "$Id: palpao_run_mirror_controller.py 25 2018-01-26 19:00:40Z lbusoni $"



def main():
    runner= Runner()
    sys.exit(runner.start(sys.argv))

if __name__ == '__main__':
    main()



#!/usr/bin/env python

import time
import sys
import signal
import os
import subprocess
import psutil
from plico.utils.base_runner import BaseRunner
from plico.utils.decorator import override
from plico.utils.logger import Logger
from palpao_server.utils.constants import Constants
from palpao_server.utils.process_startup_helper import ProcessStartUpHelper



class Runner(BaseRunner):

    RUNNING_MESSAGE = "Monitor of processes is running."

    def __init__(self):
        BaseRunner.__init__(self)

        self._logger= None
        self._processes= []
        self._timeToDie= False
        self._psh= ProcessStartUpHelper()


    def _determineInstalledBinaryDir(self):
        try:
            self._binFolder= self._configuration.getValue(
                Constants.PROCESS_MONITOR_CONFIG_SECTION,
                'binaries_installation_directory')
        except KeyError:
            self._binFolder= None


    def _logRunning(self):
        self._logger.notice(self.RUNNING_MESSAGE)
        sys.stdout.flush()


    def _setSignalIntHandler(self):
        signal.signal(signal.SIGINT, self._signalHandling)


    def _signalHandling(self, signalNumber, stackFrame):
        self._logger.notice("Received signal %d (%s)" %
                            (signalNumber, str(stackFrame)))
        if signalNumber == signal.SIGINT:
            self._timeToDie= True


    def _terminateAll(self):
        self._logger.notice("Terminating all subprocesses using psutil")
        self._logger.notice("My pid %d" % os.getpid())
        parent= psutil.Process(os.getpid())
        for process in parent.children(recursive=True):
            try:
                self._logger.notice(
                    "Killing pid %d %s" % (process.pid, process.cmdline()))
                process.send_signal(signal.SIGTERM)
            except Exception as e:
                self._logger.error("Failed killing process %s: %s" %
                                   (str(process), str(e)))


    def _spawnMirrorController(self, name):
        if self._binFolder:
            cmd= [os.path.join(self._binFolder, name)]
        else:
            cmd= [name]
        self._logger.notice("MirrorController cmd is %s" % cmd)
        mirrorController= subprocess.Popen(cmd)
        self._processes.append(mirrorController)
        return mirrorController


    def _setup(self):
        self._logger= Logger.of("Palpao process monitor runner")
        self._setSignalIntHandler()
        self._logger.notice("Creating process Mirror Controller")
        self._determineInstalledBinaryDir()
        self._mirrorController1= self._spawnMirrorController(
            Constants.MIRROR_CONTROLLER_1_PROCESS_NAME)
        self._mirrorController2= self._spawnMirrorController(
            Constants.MIRROR_CONTROLLER_2_PROCESS_NAME)


    def _runLoop(self):
        self._logRunning()
        while self._timeToDie is False:
            time.sleep(1)
        self._terminateAll()



    @override
    def run(self):
        self._setup()
        self._runLoop()
        return os.EX_OK



#!/usr/bin/env python

from plico.utils.decorator import override
from plico.utils.logger import Logger
from palpao_server.mirror_controller.abstract_deformable_mirror import \
    AbstractDeformableMirror


class SimulatedDeformableMirror(AbstractDeformableMirror):

    NUMBER_OF_ACTUATORS= 36

    def __init__(self, serialNumber):
        self._serialNumber= serialNumber
        self._logger= Logger.of('Simulated Deformable Mirror')
        self._zonalCommand= None


    @override
    def setZonalCommand(self, zonalCommand):
        assert zonalCommand.shape == (self.NUMBER_OF_ACTUATORS,), \
            "zonal command must be a vector of %d elements" % \
            self.NUMBER_OF_ACTUATORS
        self._zonalCommand= zonalCommand


    @override
    def getZonalCommand(self):
        return self._zonalCommand


    @override
    def serialNumber(self):
        return self._serialNumber


    @override
    def getNumberOfActuators(self):
        return self.NUMBER_OF_ACTUATORS


    @override
    def deinitialize(self):
        pass



import os



class ProcessStartUpHelper(object):

    def __init__(self):
        self._moduleRoot= 'palpao_server'


    def deformableMirrorStartUpScriptPath(self):
        return os.path.join(self._moduleRoot,
                            'mirror_controller',
                            'palpao_run_mirror_controller.py')


    def killAllProcessesStartUpScriptPath(self):
        return os.path.join(self._moduleRoot,
                            'utils',
                            'palpao_kill_processes.py')


    def processProcessMonitorStartUpScriptPath(self):
        return os.path.join(self._moduleRoot,
                            'process_monitor',
                            'palpao_run_process_monitor.py')


    def processProcessMonitorStopScriptPath(self):
        return os.path.join(self._moduleRoot,
                            'utils',
                            'palpao_server_stop.py')



import numpy as np
from plico.utils.decorator import override
from plico.utils.logger import Logger
from palpao_server.mirror_controller.abstract_deformable_mirror \
    import AbstractDeformableMirror


class BmcError(Exception):
    """Exception raised for Boston Micromachines Corporation error.

    Attributes:
        errorCode -- BMC error code
        message -- explanation of the error
    """

    def __init__(self, errorCode, message):
        self.errorCode = errorCode
        self.message = message


class BmcDeformableMirror(AbstractDeformableMirror):

    BMC_NO_ERR= 0

    def __init__(self, bmcDm, bmcSerialNumber):
        self._dm= bmcDm
        self._serialNumber= bmcSerialNumber
        self._logger= Logger.of('BMC Deformable Mirror')
        self._logger.notice("opening device <%s>" % self._serialNumber)
        self._openDm(self._serialNumber)
        # TODO: is this really needed? Does BmcDm really crash?
        self._initializeToZero()


    def _initializeToZero(self):
        self._logger.notice(
            "set mirror to zero. Avoid crashes"
            " when get_actuator_data is called before send_data")
        self.setZonalCommand(np.zeros(self.getNumberOfActuators()))


    def _raiseBmcError(self, errCode):
        if errCode != self.BMC_NO_ERR:
            try:
                errString= self._dm.error_string(errCode)
            except Exception:
                errString= \
                    "Unknown error message, cannot convert error code"
            raise BmcError(errCode, errString)


    def _openDm(self, serial):
        self._raiseBmcError(self._dm.open_dm(str(serial)))


    def _closeDm(self):
        self._raiseBmcError(self._dm.close_dm())


    @override
    def setZonalCommand(self, zonalCommand):
        self._raiseBmcError(self._dm.send_data(zonalCommand))


    @override
    def getZonalCommand(self):
        return np.array(self._dm.get_actuator_data())


    @override
    def serialNumber(self):
        return self._serialNumber


    @override
    def getNumberOfActuators(self):
        return self._dm.num_actuators()


    @override
    def deinitialize(self):
        self._closeDm()



from palpao_server.utils.constants import Constants


def _getDefaultConfigFilePath():
    from plico.utils.config_file_manager import ConfigFileManager
    cfgFileMgr= ConfigFileManager(Constants.APP_NAME,
                                  Constants.APP_AUTHOR,
                                  Constants.THIS_PACKAGE)
    return cfgFileMgr.getConfigFilePath()


defaultConfigFilePath= _getDefaultConfigFilePath()



#!/usr/bin/env python
import unittest
import numpy as np
from palpao_server.mirror_controller.simulated_deformable_mirror import \
    SimulatedDeformableMirror
from palpao.client.abstract_deformable_mirror_client import SnapshotEntry
from palpao.calibration.in_memory_calibration_manager import InMemoryCalibrationManager
from palpao_server.mirror_controller.deformable_mirror_controller \
    import DeformableMirrorController



class MyReplySocket():
    pass


class MyPublisherSocket():
    pass


class MyRpcHandler():

    def handleRequest(self, obj, socket, multi):
        pass


    def publishPickable(self, socket, obj):
        pass


class DeformableMirrorControllerTest(unittest.TestCase):

    def setUp(self):
        self._serverName= 'server description'
        self._ports= None
        self._dmSerialNumber= '0123456'
        self._mirror= SimulatedDeformableMirror(self._dmSerialNumber)
        self._rpcHandler= MyRpcHandler()
        self._replySocket= MyReplySocket()
        self._statusSocket= MyPublisherSocket()
        self._calMgr= InMemoryCalibrationManager()
        self._createDefaultFlat()
        self._ctrl= DeformableMirrorController(
            self._serverName,
            self._ports,
            self._mirror,
            self._replySocket,
            self._statusSocket,
            self._rpcHandler,
            self._calMgr,
            self._flatDmTag)


    def _createDefaultFlat(self):
        self._flatDmCommand= np.random.rand(
            self._mirror.getNumberOfActuators())
        self._flatDmTag='foo_foo_flatters'
        self._calMgr.saveZonalCommand(self._flatDmTag,
                                      self._flatDmCommand)


    def testGetSnapshot(self):
        snapshot= self._ctrl.getSnapshot('baar')
        serialNumberKey= 'baar.%s' % SnapshotEntry.SERIAL_NUMBER
        self.assertEqual(self._dmSerialNumber, snapshot[serialNumberKey])


    def testSetGetModalCommands(self):
        nModes= self._ctrl._getNumberOfModes()
        shapeCommands= np.arange(nModes) * 3.14
        self._ctrl.setShape(shapeCommands)
        actualShape= self._ctrl.getShape()
        self.assertTrue(np.allclose(shapeCommands, actualShape))


    def testStep(self):
        self._ctrl.step()


    def testSetFlatReferenceAtInit(self):
        wanted= self._flatDmCommand
        got= self._mirror.getZonalCommand()
        self.assertTrue(np.allclose(
            wanted, got), "%s %s" % (wanted, got))
        self.assertEqual(self._flatDmTag,
                         self._ctrl.getFlatTag())


    def testCommandsAreSummedToFlatShape(self):
        command= np.random.rand(self._ctrl._getNumberOfModes())
        self._ctrl.setShape(command)
        wanted= self._flatDmCommand + command
        got= self._mirror.getZonalCommand()
        self.assertTrue(np.allclose(
            wanted, got), "%s %s" % (wanted, got))


if __name__ == "__main__":
    unittest.main()



import abc
from plico.utils.decorator import returns, returnsNone
from six import with_metaclass



class AbstractDeformableMirror(with_metaclass(abc.ABCMeta, object)):



    @abc.abstractmethod
    @returns(str)
    def serialNumber(self):
        assert False


    @abc.abstractmethod
    def getNumberOfActuators(self):
        """ Number of actuators of the deformable mirror

        Return the number of actuators of the deformable mirror.
        number of degrees of freedom.

        Return:
            numberOfActuators (int): the number of actuators of the deformable mirror.
        """
        assert False


    @abc.abstractmethod
    @returnsNone
    def setZonalCommand(self, command):
        """ Set deformable mirror command

        Set the DM shape

        Parameters:
            command (:obj:ndarray): an array containing the required value for the actuators
                The size of the array must be equal to the number of actuators of the DM


        """
        assert False


    @abc.abstractmethod
    @returnsNone
    def getZonalCommand(self):
        """ Get Deformable Mirror command

        return the current DM command

        Return:
            shape (:obj:ndarray): an array containing the measured value for the actuators
                The size of the array must be equal to the number of actuators of the DM

        The value are measured if the DM has an internal metrology on position
        The shape sequence is taken into account.
        """
        assert False


    @abc.abstractmethod
    @returnsNone
    def deinitialize(self):
        assert False



#!/usr/bin/env python
import unittest
import numpy as np
from palpao_server.mirror_controller.pi_tip_tilt_mirror import \
    PhysikInstrumenteTipTiltMirror
from pi_gcs.fake_tip_tilt_2_axis import FakeTipTilt2Axis


class MyTipTilt2Axis(FakeTipTilt2Axis):
    pass


class PhysikInstrumenteTipTiltMirrorTest(unittest.TestCase):


    def setUp(self):
        self._serialNumber= '123123'
        self._tt= MyTipTilt2Axis()
        self._dm= PhysikInstrumenteTipTiltMirror(
            self._serialNumber,
            self._tt)


    def tearDown(self):
        self._dm.deinitialize()


    def testModulationIsStoppedAndControLoopIsDisabled(self):
        self.assertFalse(self._tt.isModulationEnabled())
        self.assertFalse(self._tt.isControlLoopEnabled())


    def testGetNumberOfActuators(self):
        self.assertEqual(2, self._dm.getNumberOfActuators())


    def testSetZonalCommand(self):
        zonalCommand= np.array([1, 2])
        self._dm.setZonalCommand(zonalCommand)
        self.assertTrue(np.allclose(
            zonalCommand,
            self._tt.getOpenLoopValue()))


if __name__ == "__main__":
    unittest.main()



from pi_gcs.tip_tilt_configuration import TipTiltConfiguration


__version__= "$Id: pi_calibration.py 189 2017-01-12 13:48:08Z pygo $"


class PhysikInstrumenteCalibration(object):

    def _createConfig(self, aLin, aOff, bLin, bOff):
        cfg= TipTiltConfiguration()
        cfg.positionToMilliRadAxisALinearCoeff= aLin
        cfg.positionToMilliRadAxisAOffsetCoeff= aOff
        cfg.positionToMilliRadAxisBLinearCoeff= bLin
        cfg.positionToMilliRadAxisBOffsetCoeff= bOff
        return cfg


    def getCalibrationFor(self, serialNumber):
        if serialNumber == 'ideal':
            return self._createConfig(0.5, -25., 0.5, -25.)
        if serialNumber == '110032051':
            return self._createConfig(0.87, -43.5, 0.87, -43.5)
        else:
            raise KeyError(
                "Calibration not available for device with serial number %s" %
                serialNumber)



import os
from palpao_server.utils.constants import Constants
from plico.utils.starter_script_creator_base import StarterScriptCreatorBase




class StarterScriptCreator(StarterScriptCreatorBase):

    def __init__(self):
        StarterScriptCreatorBase.__init__(self)


    def installExecutables(self):
        from palpao_server.utils.process_startup_helper import \
            ProcessStartUpHelper
        psh= ProcessStartUpHelper()

        self._createAStarterScript(
            os.path.join(self._binDir, Constants.START_PROCESS_NAME),
            psh.processProcessMonitorStartUpScriptPath(),
            Constants.PROCESS_MONITOR_CONFIG_SECTION
        )
        self._createAStarterScript(
            os.path.join(self._binDir,
                         Constants.MIRROR_CONTROLLER_1_PROCESS_NAME),
            psh.deformableMirrorStartUpScriptPath(),
            Constants.DEFORMABLE_MIRROR_1_CONFIG_SECTION
        )
        self._createAStarterScript(
            os.path.join(self._binDir,
                         Constants.MIRROR_CONTROLLER_2_PROCESS_NAME),
            psh.deformableMirrorStartUpScriptPath(),
            Constants.DEFORMABLE_MIRROR_2_CONFIG_SECTION
        )
        self._createAStarterScript(
            os.path.join(self._binDir, Constants.KILL_ALL_PROCESS_NAME),
            psh.killAllProcessesStartUpScriptPath(),
            'foo'
        )
        self._createAStarterScript(
            os.path.join(self._binDir, Constants.STOP_PROCESS_NAME),
            psh.processProcessMonitorStopScriptPath(),
            'not used'
        )



#!/usr/bin/env python
import logging
from plico.utils.kill_process_by_name import killProcessByName
from palpao_server.utils.constants import Constants

__version__= "$Id: palpao_stop.py 27 2018-01-27 08:48:07Z lbusoni $"



def main():
    logging.basicConfig(level=logging.INFO)
    processNames= [Constants.START_PROCESS_NAME,
                   Constants.MIRROR_CONTROLLER_1_PROCESS_NAME,
                   Constants.MIRROR_CONTROLLER_2_PROCESS_NAME,
                   ]

    for each in processNames:
        killProcessByName(each)



import abc
from plico.utils.decorator import returns, returnsForExample
import numpy as np

__version__= "$Id: abstract_modulator.py 270 2017-04-27 16:38:09Z lbusoni $"


class AbstractModulator(object, metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def name(self):
        assert False


    @abc.abstractmethod
    def setRadiusInMilliRad(self, radiusInMilliRad):
        assert False


    @abc.abstractmethod
    def setFrequencyInHz(self):
        assert False


    @abc.abstractmethod
    def setCenterInMilliRad(self, center):
        assert False


    @abc.abstractmethod
    @returns(float)
    def getRadiusInMilliRad(self):
        assert False


    @abc.abstractmethod
    @returns(float)
    def getFrequencyInHz(self):
        assert False


    @abc.abstractmethod
    @returnsForExample(np.array([-1.3, 2.3]))
    def getCenterInMilliRad(self):
        assert False


    @abc.abstractmethod
    @returnsForExample(np.zeros((9, 4000)))
    def getDiagnosticData(self):
        assert False



#!/usr/bin/env python
import os
import subprocess
import shutil
import unittest
import logging
import numpy as np
from test.test_helper import TestHelper, Poller, MessageInFileProbe,\
    ExecutionProbe
from plico.utils.configuration import Configuration
from plico.rpc.zmq_remote_procedure_call import ZmqRemoteProcedureCall
from plico.utils.logger import Logger
from palpao_server.utils.starter_script_creator import StarterScriptCreator
from palpao_server.utils.process_startup_helper import ProcessStartUpHelper
from palpao.client.deformable_mirror_client import DeformableMirrorClient
from palpao_server.mirror_controller.runner import Runner
from palpao_server.process_monitor.runner import Runner as ProcessMonitorRunner
from plico.rpc.sockets import Sockets
from plico.rpc.zmq_ports import ZmqPorts
from palpao_server.utils.constants import Constants
from palpao.client.abstract_deformable_mirror_client import SnapshotEntry



class IntegrationTest(unittest.TestCase):

    TEST_DIR= os.path.join(os.path.abspath(os.path.dirname(__file__)),
                           "./tmp/")
    LOG_DIR= os.path.join(TEST_DIR, "log")
    CONF_FILE= 'test/integration/conffiles/palpao_server.conf'
    CALIB_FOLDER= 'test/integration/calib'
    CONF_SECTION= Constants.PROCESS_MONITOR_CONFIG_SECTION
    SERVER_LOG_PATH= os.path.join(LOG_DIR, "%s.log" % CONF_SECTION)
    BIN_DIR= os.path.join(TEST_DIR, "apps", "bin")
    SOURCE_DIR= os.path.join(os.path.abspath(os.path.dirname(__file__)),
                             "../..")


    def setUp(self):
        self._setUpBasicLogging()
        self.server= None
        self._wasSuccessful= False

        self._removeTestFolderIfItExists()
        self._makeTestDir()
        self.configuration= Configuration()
        self.configuration.load(self.CONF_FILE)
        self.rpc= ZmqRemoteProcedureCall()

        calibrationRootDir= self.configuration.calibrationRootDir()
        self._setUpCalibrationTempFolder(calibrationRootDir)


    def _setUpBasicLogging(self):
        logging.basicConfig(level=logging.DEBUG)
        self._logger= Logger.of('Integration Test')


    def _makeTestDir(self):
        os.makedirs(self.TEST_DIR)
        os.makedirs(self.LOG_DIR)
        os.makedirs(self.BIN_DIR)


    def _setUpCalibrationTempFolder(self, calibTempFolder):
        shutil.copytree(self.CALIB_FOLDER,
                        calibTempFolder)


    def _removeTestFolderIfItExists(self):
        if os.path.exists(self.TEST_DIR):
            shutil.rmtree(self.TEST_DIR)


    def tearDown(self):
        TestHelper.dumpFileToStdout(self.SERVER_LOG_PATH)

        if self.server is not None:
            TestHelper.terminateSubprocess(self.server)

        if self._wasSuccessful:
            self._removeTestFolderIfItExists()


    def _createStarterScripts(self):
        ssc= StarterScriptCreator()
        ssc.setInstallationBinDir(self.BIN_DIR)
        ssc.setPythonPath(self.SOURCE_DIR)
        ssc.setConfigFileDestination(self.CONF_FILE)
        ssc.installExecutables()


    def _startProcesses(self):
        psh= ProcessStartUpHelper()
        serverLog= open(os.path.join(self.LOG_DIR, "server.out"), "wb")
        self.server= subprocess.Popen(
            [psh.processProcessMonitorStartUpScriptPath(),
             self.CONF_FILE,
             self.CONF_SECTION],
            stdout=serverLog, stderr=serverLog)
        Poller(5).check(MessageInFileProbe(
            ProcessMonitorRunner.RUNNING_MESSAGE, self.SERVER_LOG_PATH))


    def _testProcessesActuallyStarted(self):
        controllerLogFile= os.path.join(
            self.LOG_DIR,
            '%s.log' % Constants.DEFORMABLE_MIRROR_1_CONFIG_SECTION)
        Poller(5).check(MessageInFileProbe(
            Runner.RUNNING_MESSAGE, controllerLogFile))
        controller2LogFile= os.path.join(
            self.LOG_DIR,
            '%s.log' % Constants.DEFORMABLE_MIRROR_2_CONFIG_SECTION)
        Poller(5).check(MessageInFileProbe(
            Runner.RUNNING_MESSAGE, controller2LogFile))


    def _buildClients(self):
        ports1= ZmqPorts.fromConfiguration(
            self.configuration,
            Constants.DEFORMABLE_MIRROR_1_CONFIG_SECTION)
        self.deformableMirrorClient1= DeformableMirrorClient(
            self.rpc, Sockets(ports1, self.rpc))
        ports2= ZmqPorts.fromConfiguration(
            self.configuration,
            Constants.DEFORMABLE_MIRROR_2_CONFIG_SECTION)
        self.deformableMirrorClient2= DeformableMirrorClient(
            self.rpc, Sockets(ports2, self.rpc))


    def _testSetShape(self):
        numberOfModes= self.deformableMirrorClient1.getNumberOfModes()
        mirrorModalAmplitude= np.arange(numberOfModes) * 3.141
        self.deformableMirrorClient1.setShape(mirrorModalAmplitude)
        Poller(3).check(ExecutionProbe(
            lambda: self.assertTrue(
                np.allclose(
                    mirrorModalAmplitude,
                    self.deformableMirrorClient1.getShape()))))


    def _checkBackdoor(self):
        self.deformableMirrorClient1.execute(
            "import numpy as np; "
            "self._myarray= np.array([1, 2])")
        self.assertEqual(
            repr(np.array([1, 2])),
            self.deformableMirrorClient1.eval("self._myarray"))
        self.deformableMirrorClient1.execute("self._foobar= 42")
        self.assertEqual(
            "42",
            self.deformableMirrorClient1.eval("self._foobar"))


    def _testGetStatus(self):
        status= self.deformableMirrorClient1.getStatus()
        cmdCounter= status.commandCounter()
        self.deformableMirrorClient1.setShape(
            self.deformableMirrorClient1.getShape() * 2.0)
        Poller(3).check(ExecutionProbe(
            lambda: self.assertEqual(
                cmdCounter + 1,
                self.deformableMirrorClient1.getStatus().commandCounter())))


    def _testGetSnapshot(self):
        snapshot= self.deformableMirrorClient1.getSnapshot('aa')
        snKey= 'aa.%s' % SnapshotEntry.SERIAL_NUMBER
        self.assertTrue(snKey in snapshot.keys())


    def _testServerInfo(self):
        serverInfo= self.deformableMirrorClient1.serverInfo()
        self.assertEqual('ALPAO DM277 Deformable Mirror Server',
                         serverInfo.name)
        self.assertEqual('localhost', serverInfo.hostname)


    def testMain(self):
        self._buildClients()
        self._createStarterScripts()
        self._startProcesses()
        self._testProcessesActuallyStarted()
        self._testSetShape()
        self._testGetStatus()
        self._testGetSnapshot()
        self._testServerInfo()
        self._checkBackdoor()
        self._wasSuccessful= True


if __name__ == "__main__":
    unittest.main()




class FakeBmcDm():

    VALID_SERIAL_NUMBER= '0123456789AB'
    VERSION_STRING= '3.2.0'
    NUMBER_OF_ACTUATORS= 140
    NO_ERR= 0
    ERR_UNKNOWN= 1
    ERR_SERIAL_NUMBER= 4
    ERR_INVALID_ACTUATOR_COUNT= 7
    ERR_ACTUATOR_ID= 9
    ERR_NOT_OPEN= 29


    def _setupErrorMessages(self):
        self._errorMsg={
            self.NO_ERR: "All is well.",
            self.ERR_UNKNOWN: "General error.",
            self.ERR_SERIAL_NUMBER: "Invalid serial number.",
            self.ERR_INVALID_ACTUATOR_COUNT: "Invalid Number of actuators.",
            self.ERR_ACTUATOR_ID: "Incorrect actuator ID.",
            self.ERR_NOT_OPEN: "Tried to operate driver before opening."
        }


    def __init__(self):
        self._setupErrorMessages()
        self._lastErr= self.NO_ERR
        self._isOpened= False
        self._lastCommand= (0.,) * self.NUMBER_OF_ACTUATORS


    def open_dm(self, serialNumber):
        if serialNumber != self.VALID_SERIAL_NUMBER:
            self._lastErr= self.ERR_SERIAL_NUMBER
        else:
            self._isOpened= True
            self._lastErr= self.NO_ERR
        return self._lastErr


    def close_dm(self):
        self._lastErr= self.NO_ERR
        return self._lastErr


    def send_data(self, data):
        if not self._isOpened:
            self._lastErr= self.ERR_INVALID_ACTUATOR_COUNT
            return self._lastErr
        if not hasattr(data, '__iter__'):
            raise TypeError("argument must be iterable")
        if len(data) != self.NUMBER_OF_ACTUATORS:
            self._lastErr= self.ERR_INVALID_ACTUATOR_COUNT
            return self._lastErr
        else:
            self._lastCommand= tuple(data)
            self._lastErr= self.NO_ERR
        return self._lastErr


    def get_actuator_data(self):
        if not self._isOpened:
            self._lastErr= self.ERR_NOT_OPEN
            return self._lastErr
        else:
            self._lastErr= self.NO_ERR
            return self._lastCommand


    def num_actuators(self):
        if not self._isOpened:
            self._lastErr= self.NO_ERR
            return self._lastErr
        else:
            self._lastErr= self.NO_ERR
            return self.NUMBER_OF_ACTUATORS


    def poke(self):
        if not self._isOpened:
            self._lastErr= self.ERR_ACTUATOR_ID
            return self._lastErr
        else:
            self._lastErr= self.NO_ERR
            return self._lastErr


    def send_data_custom_mapping(self):
        raise Exception('method not implemented')


    def default_mapping(self):
        if not self._isOpened:
            return tuple()
        return tuple(range(self.NUMBER_OF_ACTUATORS))


    def set_segment(self):
        raise Exception('method not implemented')


    def get_segment_range(self):
        raise Exception('method not implemented')


    def load_calibration_file(self):
        raise Exception('method not implemented')


    def get_status(self):
        return self._lastErr


    def error_string(self, error_code):
        return self._errorMsg[error_code]


    def set_profiles_path(self, path):
        raise Exception('method not implemented')


    def set_maps_path(self, path):
        raise Exception('method not implemented')


    def set_calibrations_path(self, path):
        raise Exception('method not implemented')


    def configure_log(self, path, level):
        self._lastErr= self.NO_ERR
        return self._lastErr


    def version_string(self):
        return self.VERSION_STRING
